@@ -5,9 +5,9 @@
 #          config values from a given file and
 #          return it as hash structure
 #
-# Copyright (c) 2000-2022 Thomas Linden <tlinden |AT| cpan.org>.
+# Copyright (c) 2000-2025 Thomas Linden <tlinden |AT| cpan.org>.
 # All Rights Reserved. Std. disclaimer applies.
-# Artistic License, same as perl itself. Have fun.
+# Licensed under the Artistic License 2.0.
 #
 # namespace
 package Config::General;
@@ -32,11 +32,10 @@ use Carp::Heavy;
 use Carp;
 use Exporter;
 
-$Config::General::VERSION = "2.64";
+$Config::General::VERSION = "2.66";
 
-use vars  qw(@ISA @EXPORT_OK);
 use base qw(Exporter);
-@EXPORT_OK = qw(ParseConfig SaveConfig SaveConfigString);
+our @EXPORT_OK = qw(ParseConfig SaveConfig SaveConfigString);
 
 use constant _UTF8_BOM => "\x{ef}\x{bb}\x{bf}";
 
@@ -97,7 +96,8 @@ sub new {
               NormalizeValue        => 0,
               Plug                  => {},
               UseApacheIfDefine     => 0,
-              Define                => {}
+              Define                => {},
+              AlwaysQuoteOutput     => 0
              };
 
   # create the class instance
@@ -517,7 +517,9 @@ sub _open {
     # A directory was included; include all the files inside that directory in ASCII order
     local *INCLUDEDIR;
     opendir INCLUDEDIR, $configfile or croak "Config::General: Could not open directory $configfile!($!)\n";
-    my @files = sort grep { -f catfile($configfile, $_) } catfile($configfile, $_), readdir INCLUDEDIR;
+    #my @files = sort grep { -f catfile($configfile, $_) } catfile($configfile, $_), readdir INCLUDEDIR;
+    # fixes rt.cpan.org#139261
+    my @files = sort grep { -f catfile($configfile, $_) } readdir INCLUDEDIR;
     closedir INCLUDEDIR;
     local $this->{CurrentConfigFilePath} = $configfile;
     for (@files) {
@@ -833,29 +835,35 @@ sub _process_apache_ifdefine {
   my($this, $rawlines) = @_;
 
   my @filtered;
-  my $includeFlag = 1;
-  my $blockLevel  = 0;
+  my @includeFlag = (1);
 
   foreach (@{$rawlines}) {
-    if (/^<\s*IfDefine\s+([!]*)("[^"]+"|\S+)\s*>/i) {
+    if (/^\s*<\s*IfDefine\s+([!]*)("[^"]+"|\S+)\s*>/i) {
       # new IFDEF block, mark following content to be included if
       # the DEF is known, otherwise skip it til end of IFDEF
       my ($negate, $define) = ($1 eq '!',$2);
 
-      $blockLevel++;
-      $includeFlag &= ((not $negate) & (exists $this->{Define}{$define}));
-    } elsif (/^<\s*\/IfDefine\s*>/i) {
-      $blockLevel--;
-      $includeFlag = $blockLevel == 0;
-    } elsif ($includeFlag && /\s*Define\s+("[^"]+"|\S+)/i) {
+      push(@includeFlag,
+           $includeFlag[-1] & 
+           ((not $negate) & (exists $this->{Define}{$define}))
+      );
+    }
+    elsif (/^\s*<\s*\/IfDefine\s*>/i) {
+      if (scalar(@includeFlag) <= 1) {
+        croak qq(Config::General: </IfDefine> without a <IfDefine>!\n);
+      }
+      pop(@includeFlag);
+    }
+    elsif ($includeFlag[-1] && /^\s*Define\s+("[^"]+"|\S+)/i) {
       # inline Define, add it to our list
       $this->{Define}{$1} = 1;
-    } elsif ($includeFlag) {
+    }
+    elsif ($includeFlag[-1]) {
       push @filtered, $_;
     }
   }
 
-  if ($blockLevel) {
+  if (scalar(@includeFlag) > 1) {
     croak qq(Config::General: Block <IfDefine> has no EndBlock statement!\n);
   }
 
@@ -1417,11 +1425,11 @@ sub _write_scalar {
       $line =~ s/([#\$\\\"])/\\$1/g;
     }
 
-    # bugfix rt.cpan.org#42287
-    if ($line =~ /^\s/ or $line =~ /\s$/) {
-      # need to quote it
+    if ($line =~ /^\s/ || $line =~ /\s$/ || $this->{AlwaysQuoteOutput}) {
+      # quote lines containing whitespace
       $line = "\"$line\"";
     }
+
     $config_string .= $indent . $entry . $this->{StoreDelimiter} . $line . "\n";
   }
 
@@ -2112,6 +2120,11 @@ Same as B<-NormalizeBlock> but applied on options only.
 
 Same as B<-NormalizeBlock> but applied on values only.
 
+=item B<-AlwaysQuoteOutput>
+
+If set to  true, then values containing whitespace  will always quoted
+when calling C<save_string()> or C<save_file()>.
+
 =back
 
 
@@ -2846,10 +2859,10 @@ I recommend you to read the following documents, which are supplied with Perl:
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2000-2022 Thomas Linden
+Copyright (c) 2000-2025 Thomas Linden
 
 This library is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
+modify it under the same terms of the Artistic License 2.0.
 
 =head1 BUGS AND LIMITATIONS
 
@@ -2875,7 +2888,7 @@ Thomas Linden <tlinden |AT| cpan.org>
 
 =head1 VERSION
 
-2.64
+2.66
 
 =cut
 
